@@ -7,11 +7,18 @@
 
 #pragma once
 
+#include <memory>
+#include <string>
+#include <vector>
+
 #include <libcamera/base/flags.h>
+#include <libcamera/base/shared_fd.h>
 #include <libcamera/base/unique_fd.h>
 
 namespace libcamera {
 
+class FrameBuffer;
+
 class DmaBufAllocator
 {
 public:
@@ -28,13 +35,40 @@ public:
 	bool isValid() const { return providerHandle_.isValid(); }
 	UniqueFD alloc(const char *name, std::size_t size);
 
+	int exportBuffers(unsigned int count,
+			  const std::vector<unsigned int> &planeSizes,
+			  std::vector<std::unique_ptr<FrameBuffer>> *buffers);
+
 private:
+	std::unique_ptr<FrameBuffer> createBuffer(
+		std::string name, const std::vector<unsigned int> &planeSizes);
+
 	UniqueFD allocFromHeap(const char *name, std::size_t size);
 	UniqueFD allocFromUDmaBuf(const char *name, std::size_t size);
 	UniqueFD providerHandle_;
 	DmaBufAllocatorFlag type_;
 };
 
+class DmaSyncer final
+{
+public:
+	enum class SyncType {
+		Read = 0,
+		Write,
+		ReadWrite,
+	};
+
+	explicit DmaSyncer(SharedFD fd, SyncType type = SyncType::ReadWrite);
+
+	~DmaSyncer();
+
+private:
+	void sync(uint64_t step);
+
+	SharedFD fd_;
+	uint64_t flags_ = 0;
+};
+
 LIBCAMERA_FLAGS_ENABLE_OPERATORS(DmaBufAllocator::DmaBufAllocatorFlag)
 
 } /* namespace libcamera */
