@@ -21,10 +21,11 @@ class FileSink : public FrameSink
 {
 public:
 	FileSink(const libcamera::Camera *camera,
-		 const std::map<const libcamera::Stream *, std::string> &streamNames,
-		 const std::string &pattern = "");
+		 const std::map<const libcamera::Stream *, std::string> &streamNames);
 	~FileSink();
 
+	int setFilePattern(const std::string &pattern);
+
 	int configure(const libcamera::CameraConfiguration &config) override;
 
 	void mapBuffer(libcamera::FrameBuffer *buffer) override;
@@ -32,6 +33,14 @@ public:
 	bool processRequest(libcamera::Request *request) override;
 
 private:
+	static constexpr const char *kDefaultFilePattern = "frame-#.bin";
+
+	enum class FileType {
+		Binary,
+		Dng,
+		Ppm,
+	};
+
 	void writeBuffer(const libcamera::Stream *stream,
 			 libcamera::FrameBuffer *buffer,
 			 const libcamera::ControlList &metadata);
@@ -39,7 +48,10 @@ private:
 #ifdef HAVE_TIFF
 	const libcamera::Camera *camera_;
 #endif
-	std::map<const libcamera::Stream *, std::string> streamNames_;
+
 	std::string pattern_;
+	FileType fileType_;
+
+	std::map<const libcamera::Stream *, std::string> streamNames_;
 	std::map<libcamera::FrameBuffer *, std::unique_ptr<Image>> mappedBuffers_;
 };
