@@ -211,7 +211,7 @@ int MainWindow::createToolbars()
 	action->setShortcut(QKeySequence::SaveAs);
 	connect(action, &QAction::triggered, this, &MainWindow::saveImageAs);
 
-#ifdef HAVE_DNG
+#ifdef HAVE_TIFF
 	/* Save Raw action. */
 	action = toolbar_->addAction(QIcon::fromTheme("camera-photo",
 						      QIcon(":aperture.svg")),
@@ -251,16 +251,14 @@ void MainWindow::updateTitle()
 void MainWindow::switchCamera()
 {
 	/* Get and acquire the new camera. */
-	std::string newCameraId = chooseCamera();
+	std::shared_ptr<Camera> cam = chooseCamera();
 
-	if (newCameraId.empty())
+	if (!cam)
 		return;
 
-	if (camera_ && newCameraId == camera_->id())
+	if (camera_ && cam == camera_)
 		return;
 
-	const std::shared_ptr<Camera> &cam = cm_->get(newCameraId);
-
 	if (cam->acquire()) {
 		qInfo() << "Failed to acquire camera" << cam->id().c_str();
 		return;
@@ -282,40 +280,41 @@ void MainWindow::switchCamera()
 	startStopAction_->setChecked(true);
 
 	/* Display the current cameraId in the toolbar .*/
-	cameraSelectButton_->setText(QString::fromStdString(newCameraId));
+	cameraSelectButton_->setText(QString::fromStdString(cam->id()));
 }
 
-std::string MainWindow::chooseCamera()
+std::shared_ptr<Camera> MainWindow::chooseCamera()
 {
 	if (cameraSelectorDialog_->exec() != QDialog::Accepted)
-		return std::string();
+		return {};
 
-	return cameraSelectorDialog_->getCameraId();
+	std::string id = cameraSelectorDialog_->getCameraId();
+	return cm_->get(id);
 }
 
 int MainWindow::openCamera()
 {
-	std::string cameraName;
-
 	/*
-	 * Use the camera specified on the command line, if any, or display the
-	 * camera selection dialog box otherwise.
+	 * If a camera is specified on the command line, get it. Otherwise, if
+	 * only one camera is available, pick it automatically, else, display
+	 * the selector dialog box.
 	 */
-	if (options_.isSet(OptCamera))
-		cameraName = static_cast<std::string>(options_[OptCamera]);
-	else
-		cameraName = chooseCamera();
-
-	if (cameraName == "")
-		return -EINVAL;
-
-	/* Get and acquire the camera. */
-	camera_ = cm_->get(cameraName);
-	if (!camera_) {
-		qInfo() << "Camera" << cameraName.c_str() << "not found";
-		return -ENODEV;
+	if (options_.isSet(OptCamera)) {
+		std::string cameraName = static_cast<std::string>(options_[OptCamera]);
+		camera_ = cm_->get(cameraName);
+		if (!camera_)
+			qInfo() << "Camera" << cameraName.c_str() << "not found";
+	} else {
+		std::vector<std::shared_ptr<Camera>> cameras = cm_->cameras();
+		camera_ = (cameras.size() == 1) ? cameras[0] : chooseCamera();
+		if (!camera_)
+			qInfo() << "No camera detected";
 	}
 
+	if (!camera_)
+		return -ENODEV;
+
+	/* Acquire the camera. */
 	if (camera_->acquire()) {
 		qInfo() << "Failed to acquire camera";
 		camera_.reset();
@@ -323,7 +322,7 @@ int MainWindow::openCamera()
 	}
 
 	/* Set the camera switch button with the currently selected Camera id. */
-	cameraSelectButton_->setText(QString::fromStdString(cameraName));
+	cameraSelectButton_->setText(QString::fromStdString(camera_->id()));
 
 	return 0;
 }
@@ -646,7 +645,7 @@ void MainWindow::captureRaw()
 void MainWindow::processRaw(FrameBuffer *buffer,
 			    [[maybe_unused]] const ControlList &metadata)
 {
-#ifdef HAVE_DNG
+#ifdef HAVE_TIFF
 	QString defaultPath = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
 	QString filename = QFileDialog::getSaveFileName(this, "Save DNG", defaultPath,
 							"DNG Files (*.dng)");
