@@ -8,7 +8,7 @@
 #include <assert.h>
 
 #include "cam_helper.h"
-#include "math.h"
+
 using namespace RPiController;
 
 class CamHelperImx283 : public CamHelper
@@ -17,8 +17,6 @@ public:
 	CamHelperImx283();
 	uint32_t gainCode(double gain) const override;
 	double gain(uint32_t gainCode) const override;
-	void getDelays(int &exposureDelay, int &gainDelay,
-		       int &vblankDelay, int &hblankDelay) const override;
 	unsigned int hideFramesModeSwitch() const override;
 
 private:
@@ -49,16 +47,6 @@ double CamHelperImx283::gain(uint32_t gainCode) const
 	return static_cast<double>(2048.0 / (2048 - gainCode));
 }
 
-void CamHelperImx283::getDelays(int &exposureDelay, int &gainDelay,
-				int &vblankDelay, int &hblankDelay) const
-{
-	/* The driver appears to behave as follows: */
-	exposureDelay = 2;
-	gainDelay = 2;
-	vblankDelay = 2;
-	hblankDelay = 2;
-}
-
 unsigned int CamHelperImx283::hideFramesModeSwitch() const
 {
 	/* After a mode switch, we seem to get 1 bad frame. */
