@@ -30,7 +30,7 @@ struct AgcMeteringMode {
 };
 
 struct AgcExposureMode {
-	std::vector<libcamera::utils::Duration> shutter;
+	std::vector<libcamera::utils::Duration> exposureTime;
 	std::vector<double> gain;
 	int read(const libcamera::YamlObject &params);
 };
@@ -90,8 +90,8 @@ public:
 	std::vector<double> const &getWeights() const;
 	void setEv(double ev);
 	void setFlickerPeriod(libcamera::utils::Duration flickerPeriod);
-	void setMaxShutter(libcamera::utils::Duration maxShutter);
-	void setFixedShutter(libcamera::utils::Duration fixedShutter);
+	void setMaxExposureTime(libcamera::utils::Duration maxExposureTime);
+	void setFixedExposureTime(libcamera::utils::Duration fixedExposureTime);
 	void setFixedAnalogueGain(double fixedAnalogueGain);
 	void setMeteringMode(std::string const &meteringModeName);
 	void setExposureMode(std::string const &exposureModeName);
@@ -117,7 +117,7 @@ private:
 	bool applyDigitalGain(double gain, double targetY, bool channelBound);
 	void divideUpExposure();
 	void writeAndFinish(Metadata *imageMetadata, bool desaturate);
-	libcamera::utils::Duration limitShutter(libcamera::utils::Duration shutter);
+	libcamera::utils::Duration limitExposureTime(libcamera::utils::Duration exposureTime);
 	double limitGain(double gain) const;
 	AgcMeteringMode *meteringMode_;
 	AgcExposureMode *exposureMode_;
@@ -128,7 +128,7 @@ private:
 	struct ExposureValues {
 		ExposureValues();
 
-		libcamera::utils::Duration shutter;
+		libcamera::utils::Duration exposureTime;
 		double analogueGain;
 		libcamera::utils::Duration totalExposure;
 		libcamera::utils::Duration totalExposureNoDG; /* without digital gain */
@@ -146,8 +146,8 @@ private:
 	std::string constraintModeName_;
 	double ev_;
 	libcamera::utils::Duration flickerPeriod_;
-	libcamera::utils::Duration maxShutter_;
-	libcamera::utils::Duration fixedShutter_;
+	libcamera::utils::Duration maxExposureTime_;
+	libcamera::utils::Duration fixedExposureTime_;
 	double fixedAnalogueGain_;
 };
 
