@@ -336,7 +336,7 @@ void CameraConfiguration::addConfiguration(const StreamConfiguration &cfg)
  * \retval CameraConfiguration::Invalid The configuration is invalid and can't
  * be adjusted. This may only occur in extreme cases such as when the
  * configuration is empty.
- * \retval CameraConfigutation::Adjusted The configuration has been adjusted
+ * \retval CameraConfiguration::Adjusted The configuration has been adjusted
  * and is now valid. Parameters may have changed for any stream, and stream
  * configurations may have been removed. The caller shall check the
  * configuration carefully.
@@ -603,6 +603,11 @@ Camera::Private::~Private()
  * \return The pipeline handler that created this camera
  */
 
+/**
+ * \fn Camera::Private::pipe() const
+ * \copydoc Camera::Private::pipe()
+ */
+
 /**
  * \fn Camera::Private::validator()
  * \brief Retrieve the control validator related to this camera
@@ -1178,8 +1183,8 @@ int Camera::configure(CameraConfiguration *config)
 	if (ret < 0)
 		return ret;
 
-	for (auto it : *config)
-		it.setStream(nullptr);
+	for (auto &cfg : *config)
+		cfg.setStream(nullptr);
 
 	if (config->validate() != CameraConfiguration::Valid) {
 		LOG(Camera, Error)
